import os
import csv
import jinja2
import argparse
import xml.etree.cElementTree as element


# TODO : add *.tex support
# TODO : add *.xls support


DEFAULT_SCRIPT_PATH   = os.path.dirname(os.path.abspath(__file__))
DEFAULT_TEMPLATE_PATH = os.path.join(DEFAULT_SCRIPT_PATH, 'templates')
DEFAULT_RESULTS_PATH  = os.path.join(DEFAULT_SCRIPT_PATH, 'results')
DEFAULT_CONFIG_PATH   = os.path.join(DEFAULT_SCRIPT_PATH, 'configuration')


class BaseTableGenerator(object):
    """
    Class that implements table generator interface

    :Attributes:
        template_path (string) : full-path to a directory with html templates
        results_path  (string) : full-path to a results path
        config_path   (string) : full-path to a configuration file

    :Properties:
        None

    :Methods:
        generate(self, context, template) : <interface> generate the table

    """

    def __init__(self, *args, **kwargs):
        """
        Initialize TableGenerator instance

        :Attributes:
            template_path (string) : full-path to a directory with html templates
            results_path  (string) : full-path to a results path
            config_path   (string) : full-path to a configuration file

        """

        # Parse input keyword arguments
        self.template_path = kwargs.get('template_path', None)
        self.results_path  = kwargs.get('results_path', None)
        self.config_path   = kwargs.get('config_path', None)

        # Handle None-valued arguments
        if self.template_path is None:
            self.template_path = DEFAULT_TEMPLATE_PATH
        if self.results_path is None:
            self.results_path = DEFAULT_RESULTS_PATH
        if self.config_path is None:
            self.config_path = DEFAULT_CONFIG_PATH

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return "{}\n{}\n{}\n".format(
            self.template_path,
            self.results_path,
            self.config_path
        )

    def generate(self, context, template):
        """
        Generate the table interface

        :Attributes:
            context  (list)   : list of context dicts
            template (string) : full-path to a template file

        :Returns:
            None

        """

        # Raise not implemented exception
        raise NotImplementedError('A subclass must override the method.')


class HTMLTableGenerator(BaseTableGenerator):
    """
    Class that implements HTML table generator routines

    :Attributes:
        template_path (string) : full-path to a directory with html templates
        results_path  (string) : full-path to a results path (*.html)
        config_path   (string) : full-path to a configuration file (*.json)

    :Properties:
        None

    :Methods:
        generate(self, context, template) : generate the HTML table

    """

    def __init__(self, *args, **kwargs):
        """
        Initialize HTMLTableGenerator instance

        :Attributes:
            template_path (string) : full-path to a directory with html templates
            results_path  (string) : full-path to a results path (*.html)
            config_path   (string) : full-path to a configuration file (*.json)

        """

        # Instantiate the super class object
        super().__init__(*args, **kwargs)

        # Initialize jinja environment
        if self.template_path:
            self.environment = jinja2.Environment(
                loader=jinja2.FileSystemLoader(self.template_path),
                autoescape=False,
                trim_blocks=False
            )

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return "{}\n{}\n".format(self.template_path, self.config_path)

    def generate(self, context, template):
        """
        Generate the HTML table

        :Attributes:
            context  (list)   : list of context dicts
            template (string) : full-path to a template file (*.HTML)

        :Returns:
            None

        """

        # Render the HTML template
        if template in os.listdir(self.template_path):
            with open(os.path.join(self.results_path, template), 'w') as _file:
                _file.write(self.environment.get_template(template).render(context=context))


class XMLTableGenerator(BaseTableGenerator):
    """
    Class that implements XML table generator routines

    :Attributes:
        template_path (string) : full-path to a directory with html templates
        results_path  (string) : full-path to a results path (*.xml)
        config_path   (string) : full-path to a configuration file (*.json)

    :Properties:
        None

    :Methods:
        generate(self, context, template) : generate the XML table

    """

    def __init__(self, *args, **kwargs):
        """
        Initialize XMLTableGenerator instance

        :Attributes:
            template_path (string) : full-path to a directory with html templates
            results_path  (string) : full-path to a results path (*.xml)
            config_path   (string) : full-path to a configuration file (*.json)

        """

        # Instantiate the super class object
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return "{}\n{}\n".format(self.template_path, self.config_path)

    def generate(self, context, template):
        """
        Generate the XML table

        :Attributes:
            context  (list)   : list of context dicts
            template (string) : full-path to a template file (*.XML)

        :Returns:
            None

        """

        # Create the XML file
        results = element.Element("results")

        for _data in context:
            result = element.SubElement(results, "result")
            element.SubElement(result, "true_label").text = str(_data.true_label)
            element.SubElement(result, "pred_label").text = str(_data.pred_label)
            element.SubElement(result, "model_spec").text = str(_data.model_spec)
            element.SubElement(result, "mae").text = str(_data.mae)
            element.SubElement(result, "mse").text = str(_data.mse)
            element.SubElement(result, "rmse").text = str(_data.rmse)

        tree = element.ElementTree(results)
        tree.write(os.path.join(self.results_path, template))


class CSVTableGenerator(BaseTableGenerator):
    """
    Class that implements CSV table generator routines

    :Attributes:
        template_path (string) : full-path to a directory with html templates
        results_path  (string) : full-path to a results path (*.csv)
        config_path   (string) : full-path to a configuration file (*.json)

    :Properties:
        None

    :Methods:
        generate(self, context, template) : generate the CSV table

    """

    def __init__(self, *args, **kwargs):
        """
        Initialize CSVTableGenerator instance

        :Attributes:
            template_path (string) : full-path to a directory with html templates
            results_path  (string) : full-path to a results path (*.csv)
            config_path   (string) : full-path to a configuration file (*.json)

        """

        # Instantiate the super class object
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return self.__class__.__name__

    def __str__(self):
        return "{}\n{}\n".format(self.template_path, self.config_path)

    def generate(self, context, template):
        """
        Generate the CSV table

        :Attributes:
            context  (list)   : list of context dicts
            template (string) : full-path to a template file (*.CSV)

        :Returns:
            None

        """

        # Create the CSV file
        with open(os.path.join(self.results_path, template), 'w+') as _file:
            for _data in context:
                _row = csv.writer(_file, quoting=csv.QUOTE_ALL)
                _row.writerow(_data.items())


if __name__ == "__main__":

    # Instantiate the parser
    _parser = argparse.ArgumentParser()

    # Add optional command line arguments
    _parser.add_argument('--template_path', type=str, help='template file path')
    _parser.add_argument('--results_path', type=str, help='results file path')
    _parser.add_argument('--config_path', type=str, help='config file path')

    # Parse the command line arguments
    _args = _parser.parse_args()

    # TODO : add specific class instantiation
    # TODO : add specific method call parser
